!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!                                                                                       !
!                        Heat exchange sub-model of ELCIRC                              !
!                           Version 8 (October 22, 2004)                                !
!                                                                                       !
!                 Center for Coastal and Land-Margin Research                           !
!             Department of Environmental Science and Engineering                       !
!                   OGI School of Science and Engineering,                              !
!                     Oregon Health & Science University                                !
!                       Beaverton, Oregon 97006, USA                                    !
!                                                                                       !
!                   Scientific direction: Antonio Baptista                              !
!                   Code development: Mike A. Zulauf                  			!
!                                                                                       !
!               Copyright 1999-2004 Oregon Health and Science University                !
!                              All Rights Reserved                                      !
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
!  
!
!-----------------------------------------------------------------------
! Note: the following global variables (from module global) are used in
!       this code.  This list does not include variables passed in as
!       arguments. . .
!
!       mnp
!       np
!       uu2
!       vv2
!       tnd
!       snd
!       kfp
!       idry
!       nvrt
!       ivcor
!-----------------------------------------------------------------------
!
! Note that this code depends on the existence of two additional
! subroutines:
!   get_wind (called from within ELCIRC)
!   get_rad  (called from within surf_fluxes)
!
!-----------------------------------------------------------------------
!
! Note that net downward heat flux (except for solar) is given by:
!
! net_sfc_flux_d = - (sen_flux + lat_flux + (longwave_u - longwave_d))
!
      subroutine surf_fluxes (time, u_air, v_air, p_air, &
     &                   t_air, q_air, shortwave_d, &
     &                   sen_flux, lat_flux, longwave_u, longwave_d, &
     &                   tau_xz, tau_yz)

!       implicit none
        use global, only : mnp, np, uu2, vv2, tnd, snd, &
     &                     kfp, idry, nvrt, ivcor
        implicit real*8(a-h,o-z),integer(i-n)

! define some new names for things in header file
        integer max_nodes
        parameter (max_nodes = mnp)

! input/output variables
        real*8 u_air(max_nodes), v_air(max_nodes)
        real*8 t_air(max_nodes), q_air(max_nodes)
        real*8 p_air(max_nodes), time
        real*8 sen_flux(max_nodes), lat_flux(max_nodes)
        real*8 longwave_d(max_nodes), longwave_u(max_nodes)
        real*8 shortwave_d(max_nodes)
        real*8 tau_xz(max_nodes), tau_yz(max_nodes)

! local variables
        integer num_nodes, i_node, sfc_lev
        logical dry
        real*8 one_third, stefan, t_freeze, emissivity
        parameter (stefan = 5.67e-8)
        parameter (t_freeze = 273.15)
        parameter (emissivity = 1.0)
        integer printit

        printit = 1000

! define the local variables num_nodes
        num_nodes = np

        open(38,file='fort.38')
        rewind(38)
        write(38,*)
        write(38,*) 'enter surf_fluxes'

! define some constants, etc
        one_third = 1.0 / 3.0

! retrieve the downwelling radiative fluxes
        call get_rad (time, shortwave_d, longwave_d)

! output info to debug file
        write(38,*) 'num_nodes = ', num_nodes
        write(38,*) 'time      = ', time

! convert air temperatures to Celcius
        do i_node = 1, num_nodes
          t_air(i_node) = t_air(i_node) - t_freeze

! specify the surface level at this node (depends on coordinate system)
          if (ivcor .eq. -1) then         ! z
            sfc_lev = kfp(i_node)
          else                            ! sigma
            sfc_lev = nvrt
          endif

          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*)
            write(38,*) 'i_node, sfc u, v, T = ', i_node, &
#ifndef SELFE
     &                  uu2(i_node, sfc_lev), &
     &                  vv2(i_node, sfc_lev), &
     &                  tnd(i_node, sfc_lev)
#else /* SELFE */
     &                  uu2(sfc_lev,i_node), &
     &                  vv2(sfc_lev,i_node), &
     &                  tnd(sfc_lev,i_node)
#endif /* SELFE */
            write(38,*) 'u, v, p, T, q (air) = ', u_air(i_node), &
     &                  v_air(i_node), p_air(i_node), t_air(i_node), &
     &                  q_air(i_node)
          endif

        enddo

! calculate the turbulent fluxes at the nodes
        write(38,*) 'above turb_fluxes'
        call turb_fluxes (num_nodes, &
     &                    u_air, v_air, p_air, t_air, q_air, &
     &                    sen_flux, lat_flux, tau_xz, tau_yz)
        write(38,*) 'below turb_fluxes'

! now calculate upwards longwave flux at the surface, using black-body
! equation
        write(38,*) 'calculating longwave_u'
        do i_node = 1, num_nodes

! specify the surface level at this node (depends on coordinate system)
          if (ivcor .eq. -1) then         ! z
            sfc_lev = kfp(i_node)
          else                            ! sigma
            sfc_lev = nvrt
          endif

          longwave_u(i_node) = emissivity * stefan * &
#ifndef SELFE
     &                      ( t_freeze + tnd(i_node, sfc_lev) ) ** 4
#else /* SELFE */
     &                      ( t_freeze + tnd(sfc_lev,i_node) ) ** 4
#endif /* SELFE */

        enddo

        do i_node = 1, num_nodes
          if (mod(i_node-1,printit) .eq. 0) then

! define whether this node is dry or not (depends on coordinate system)
            dry = &
     &          ( (ivcor .eq. -1) .and. (kfp(i_node)  .eq. 0) ) & ! z
     &        .or. &
     &          ( (ivcor .ne. -1) .and. (idry(i_node) .eq. 1) )   !sigma

            if (.not. dry) then
              write(38,*)
              write(38,*) 'i_node = ', i_node
              write(38,*) 'net_sfc_flux_d = ', &
     &                     - sen_flux(i_node) - lat_flux(i_node) &
     &                     - ( longwave_u(i_node) - longwave_d(i_node) )
              write(38,*) 'shortwave_d = ', shortwave_d(i_node)
              write(38,*) 'longwave_d, longwave_u = ', &
     &                     longwave_d(i_node), longwave_u(i_node)
              write(38,*) 'sen_flux, lat_flux = ', &
     &                     sen_flux(i_node), lat_flux(i_node)
            else
              write(38,*)
              write(38,*) 'i_node = ', i_node
              write(38,*) 'dry!'
            endif
          endif
        enddo

        close(38)

      return
      end
!-----------------------------------------------------------------------
!
! Calculate bulk aerodynamic surface fluxes over water using method of
! Zeng et al., J Clim, v11, p 2628, Oct 1998.
!
! Note: this subroutine uses actual temperatures instead of potential
!       temperatures.  Since the temperature height is only 2m, the
!       difference should be negligible. . .
!
      subroutine turb_fluxes (num_nodes, &
     &                        u_air, v_air, p_air, t_air, q_air, &
     &                        sen_flux, lat_flux, tau_xz, tau_yz)

!       implicit none
        use global, only : mnp, uu2, vv2, tnd, snd, &
     &                     kfp, idry, nvrt, ivcor
        implicit real*8(a-h,o-z),integer(i-n)

! define some new names for things in header file
        integer max_nodes
        parameter (max_nodes = mnp)

! input/output variables
        integer num_nodes
        real*8 u_air(max_nodes), v_air(max_nodes), p_air(max_nodes)
        real*8 t_air(max_nodes), q_air(max_nodes)
        real*8 sen_flux(max_nodes), lat_flux(max_nodes)
        real*8 tau_xz(max_nodes), tau_yz(max_nodes)

! local variables
        integer i_node, iter, max_iter, sfc_lev
        parameter (max_iter = 10)
        real*8 u_star, theta_star, q_star, z_0, monin
        real*8 zeta_u, zeta_t
        real*8 one_third, two_thirds, z_t, z_u
        real*8 a1, a2, b1, b2, nu, g, beta, w_star, mix_ratio
        real*8 t_v, z_i, speed, karman, zeta_m, psi_m, zeta_h, psi_h
        real*8 re, z_0_t, e_sfc, t_freeze, q_sfc, epsilon_r
        real*8 rho_air, c_p_air, latent, r_air, rb
        real*8 theta_air, theta_v_air, delta_theta, delta_q
        real*8 delta_theta_v, theta_v_star, speed_res, tau
        real*8 speed_air, speed_water, speed_air_warn, speed_air_stop
        real*8 speed_water_warn, speed_water_stop
        parameter (speed_air_warn = 50.0)
        parameter (speed_air_stop = 100.0)
        parameter (speed_water_warn = 5.0)
        parameter (speed_water_stop = 10.0)
        real*4 esat_flat_r
        parameter (z_t = 2.0)
        parameter (z_u = 10.0)
        parameter (a1 = 0.013)
        parameter (a2 = 0.11)
        parameter (b1 = 2.67)
        parameter (b2 = -2.57)
        parameter (nu = 1.46e-5)
        parameter (beta = 1.0)
        parameter (g = 9.81)
        parameter (z_i = 1000.0)
        parameter (karman = 0.4)
        parameter (zeta_m = -1.574)
        parameter (zeta_h = -0.465)
        parameter (t_freeze = 273.15)
        parameter (epsilon_r = 0.6220)
        parameter (c_p_air = 1004.0)
        parameter (latent = 2.501e6)
        parameter (r_air = 287.0)
        integer printit
        logical converged, dry
        printit = 1000

        write(38,*) 'enter turb_fluxes'

! precalculate constants
        one_third = 1.0 / 3.0
        two_thirds = 2.0 / 3.0

! now loop over all points
        do i_node = 1, num_nodes
          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*)
            write(38,*) 'i_node = ', i_node
          endif

! define whether this node is dry or not (depends on coordinate system)
          dry = &
     &        ( (ivcor .eq. -1) .and. (kfp(i_node)  .eq. 0) ) & ! z
     &      .or. &
     &        ( (ivcor .ne. -1) .and. (idry(i_node) .eq. 1) )   !sigma

! if this point isn't dry, then calculate fluxes (if dry, then skip)
        if (.not. dry) then

! specify the surface level at this node (depends on coordinate system)
          if (ivcor .eq. -1) then         ! z
            sfc_lev = kfp(i_node)
          else                            ! sigma
            sfc_lev = nvrt
          endif

! calculate q_sfc from e_sfc
! (e_sfc reduced for salinity using eqn from Smithsonian Met Tables)
#ifndef SELFE
          e_sfc = (1.0 - 0.000537 * snd(i_node, sfc_lev)) &
     &          * esat_flat_r(real(tnd(i_node, sfc_lev) + t_freeze))
#else /* SELFE */
          e_sfc = (1.0 - 0.000537 * snd(sfc_lev,i_node)) &
     &          * esat_flat_r(real(tnd(sfc_lev,i_node) + t_freeze))
#endif /* SELFE */
          q_sfc = epsilon_r * e_sfc &
     &          / ( p_air(i_node) - e_sfc * (1.0 - epsilon_r) )

! calculate the water vapor mixing ratio of the air
          mix_ratio = q_air(i_node) / (1.0 - q_air(i_node))

! calculate theta_air, theta_v_air, delta_theta, delta_q,
! and delta_theta_v
          theta_air = (t_air(i_node) + t_freeze) + 0.0098*z_t
          theta_v_air = theta_air * (1.0 + 0.608 * mix_ratio)
          delta_theta = theta_air - &
#ifndef SELFE
     &                  (tnd(i_node, sfc_lev) + t_freeze)
#else /* SELFE */
     &                  (tnd(sfc_lev,i_node) + t_freeze)
#endif /* SELFE */
          delta_q = q_air(i_node) - q_sfc
          delta_theta_v = delta_theta * (1.0 + 0.608 * mix_ratio) &
     &                  + 0.608 * theta_air * delta_q

! calculate the air virtual temperature and density
          t_v = (t_air(i_node) + t_freeze) * (1.0 + 0.608 * mix_ratio)
          rho_air = p_air(i_node) / (r_air * t_v)
          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*) 'e_sfc, q_sfc, mix_ratio = ', &
     &                   e_sfc, q_sfc, mix_ratio
            write(38,*) 'theta_air, theta_v_air, delta_theta = ', &
     &                   theta_air, theta_v_air, delta_theta
            write(38,*) 'delta_q, delta_theta_v, t_v = ', &
     &                   delta_q, delta_theta_v, t_v
            write(38,*) 'rho_air = ', rho_air
          endif

! check air and water speeds, and give warnings and/or bombs for
! excessive values
          speed_air = sqrt( u_air(i_node)*u_air(i_node) + &
     &                      v_air(i_node)*v_air(i_node) )
          speed_water &
#ifndef SELFE
     &      = sqrt( uu2(i_node, sfc_lev)*uu2(i_node, sfc_lev) + &
     &              vv2(i_node, sfc_lev)*vv2(i_node, sfc_lev) )
#else /* SELFE */
     &      = sqrt( uu2(sfc_lev,i_node)*uu2(sfc_lev,i_node) + &
     &              vv2(sfc_lev,i_node)*vv2(sfc_lev,i_node) )
#endif /* SELFE */

          if (speed_air .gt. speed_air_stop) then
            write(11,*)
            write(11,*) 'speed_air exceeds ', speed_air_stop
            stop
          else if (speed_air .gt. speed_air_warn) then
            write(12,*)
            write(12,*) 'speed_air exceeds ', speed_air_warn
          endif

          if (speed_water .gt. speed_water_stop) then
            write(11,*)
            write(11,*) 'speed_water exceeds ', speed_water_stop
            stop
          else if (speed_water .gt. speed_water_warn) then
            write(12,*)
            write(12,*) 'speed_water exceeds ', speed_water_warn
          endif

! begin with initial values of u_star, w_star, and speed
          u_star = 0.06
          w_star = 0.5
          if (delta_theta_v .ge. 0) then                    ! stable
            speed = &
     &        max( sqrt( &
#ifndef SELFE
     &               (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &               (v_air(i_node) - vv2(i_node, sfc_lev))**2 ), &
#else /* SELFE */
     &               (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &               (v_air(i_node) - vv2(sfc_lev,i_node))**2 ), &
#endif /* SELFE */
     &             0.1d0)
          else                                              ! unstable
            speed = &
#ifndef SELFE
     &        sqrt( (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &              (v_air(i_node) - vv2(i_node, sfc_lev))**2 + &
#else /* SELFE */
     &        sqrt( (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &              (v_air(i_node) - vv2(sfc_lev,i_node))**2 + &
#endif /* SELFE */
     &              (beta * w_star)**2 )
          endif

! now loop to obtain good initial values for u_star and z_0
          do iter = 1, 5
            z_0 = a1 * u_star * u_star / g + a2 * nu / u_star
            u_star = karman * speed / log(z_u/z_0)
          enddo

! calculate rb (some stability parameter from Xubin's code?)
          rb = g * z_u * delta_theta_v / (theta_v_air * speed * speed)

! calculate initial values for zeta_u, monin, zeta_t
          if (rb .ge. 0) then                      ! neutral or stable
            zeta_u = rb * log(z_u/z_0) &
     &             / (1.0 - 0.5*min(rb,0.19d0))
          else
            zeta_u = rb * log(z_u/z_0)
          endif
          monin = z_u / zeta_u
          zeta_t = z_t / monin
          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*) 'speed, z_0, u_star = ', &
     &                   speed, z_0, u_star
            write(38,*) 'zeta_u, zeta_t, monin = ', &
     &                   zeta_u, zeta_t, monin
          endif

! iterate a maximum of max_iter times
          iter = 0
          converged = .false.
100       continue
            iter = iter + 1

! Calculate the roughness lengths
            z_0 = a1 * u_star * u_star / g + a2 * nu / u_star
            re = u_star * z_0 / nu
            z_0_t = z_0 / exp(b1 * (re**0.25) + b2)
            if (mod(i_node-1,printit) .eq. 0) then
              write(38,*) 're, z_0, z_0_t = ', &
     &                     re, z_0, z_0_t
            endif

! calculate the zetas
            zeta_u = z_u / monin
            zeta_t = z_t / monin

! apply asymptotic limit to stable conditions
            if (zeta_t .gt. 2.5) then
              converged = .true.
              zeta_t = 2.5
              monin = z_t / zeta_t
              zeta_u = z_u / monin
              if (mod(i_node-1,printit) .eq. 0) then
                write(38,*) 'limiting zeta_u, zeta_t, monin!'
              endif
            endif

! caulculate u_star, depending on zeta
            if (zeta_u .lt. zeta_m) then                ! very unstable

              u_star = speed * karman &
     &               / ( log(zeta_m*monin/z_0) &
     &                   - psi_m(zeta_m) &
! extra term?
     &                   + psi_m(z_0/monin) &
     &                   + 1.14 * ((-zeta_u)**(one_third) - &
     &                           (-zeta_m)**(one_third)) )

            else if (zeta_u .lt. 0.0) then              ! unstable

              u_star = speed * karman &
     &               / ( log(z_u/z_0) &
     &                   - psi_m(zeta_u) &
! extra term?
     &                   + psi_m(z_0/monin) &
     &                 )

            else if (zeta_u .le. 1.0) then              ! neutral/stable

              u_star = speed * karman &
     &               / ( log(z_u/z_0) + 5.0*zeta_u &
! extra term?
     &                   - 5.0*z_0/monin &
     &                 )

            else                                        ! very stable

              u_star = speed * karman &
     &               / ( log(monin/z_0) + 5.0 &
     &                   + 5.0*log(zeta_u) &
! extra term?
     &                   - 5.0*z_0/monin &
     &                   + zeta_u - 1.0 )

            endif

! caulculate theta_star and q_star, depending on zeta
            if (zeta_t .lt. zeta_h) then                ! very unstable

              theta_star = karman * delta_theta &
     &                   / ( log(zeta_h*monin/z_0_t) &
     &                       - psi_h(zeta_h) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                       + 0.8 * ((-zeta_h)**(-one_third) - &
     &                                (-zeta_t)**(-one_third)) )

              q_star = karman * delta_q &
     &                   / ( log(zeta_h*monin/z_0_t) &
     &                       - psi_h(zeta_h) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                       + 0.8 * ((-zeta_h)**(-one_third) - &
     &                                (-zeta_t)**(-one_third)) )

            else if (zeta_t .lt. 0.0) then              ! unstable

              theta_star = karman * delta_theta &
     &                   / ( log(z_t/z_0_t) &
     &                       - psi_h(zeta_t) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                     )

              q_star = karman * delta_q &
     &                   / ( log(z_t/z_0_t) &
     &                       - psi_h(zeta_t) &
! extra term?
     &                       + psi_h(z_0_t/monin) &
     &                     )

            else if (zeta_t .lt. 1.0) then              ! neutral/stable

              theta_star = karman * delta_theta &
     &                   / ( log(z_t/z_0_t) &
     &                       + 5.0*zeta_t &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                     )

              q_star = karman * delta_q &
     &                   / ( log(z_t/z_0_t) &
     &                       + 5.0*zeta_t &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                     )

            else                                        ! very stable

              theta_star = karman * delta_theta &
     &                   / ( log(monin/z_0_t) + 5.0 &
     &                       + 5.0*log(zeta_t) &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                       + zeta_t - 1.0 )

              q_star = karman * delta_q &
     &                   / ( log(monin/z_0_t) + 5.0 &
     &                       + 5.0*log(zeta_t) &
! extra term?
     &                       - 5.0*z_0_t/monin &
     &                       + zeta_t - 1.0 )

            endif

! calculate theta_v_star and monin
            theta_v_star = theta_star * (1.0 + 0.608 * mix_ratio) &
     &                   + 0.608 * theta_air * q_star
            monin = theta_v_air * u_star * u_star &
     &            / (karman * g * theta_v_star)

! depending on surface layer stability, calculate the effective
! near-surface wind speed
! (ie relative to the flowing water surface)
            if (delta_theta_v .ge. 0.0) then                  ! stable

              speed = &
     &          max( sqrt( &
#ifndef SELFE
     &                 (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &                 (v_air(i_node) - vv2(i_node, sfc_lev))**2 ), &
#else /* SELFE */
     &                 (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &                 (v_air(i_node) - vv2(sfc_lev,i_node))**2 ), &
#endif /* SELFE */
     &               0.1d0)

            else                                              ! unstable

! calculate the convective velocity scale
              w_star = (-g * theta_v_star * u_star * z_i / theta_v_air) &
     &                 ** one_third

              speed = &
#ifndef SELFE
     &          sqrt( (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &                (v_air(i_node) - vv2(i_node, sfc_lev))**2 + &
#else /* SELFE */
     &          sqrt( (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &                (v_air(i_node) - vv2(sfc_lev,i_node))**2 + &
#endif /* SELFE */
     &                (beta * w_star)**2 )

            endif
            if (mod(i_node-1,printit) .eq. 0) then
              write(38,*) 'iter, u_star, q_star, theta_star = ', &
     &                     iter, u_star, q_star, theta_star
              write(38,*) 'iter, theta_v_star, monin, speed = ', &
     &                     iter, theta_v_star, monin, speed
              write(38,*) 'iter, zeta_u, zeta_t = ', &
     &                     iter, zeta_u, zeta_t
            endif

! bottom of main iteration loop
          if (.not. converged .and. iter .lt. max_iter) goto 100

! calculate fluxes
          sen_flux(i_node) = - rho_air * c_p_air * u_star * theta_star
          lat_flux(i_node) = - rho_air * latent * u_star * q_star

! calculate wind stresses
          speed_res = &
#ifndef SELFE
     &          sqrt( (u_air(i_node) - uu2(i_node, sfc_lev))**2 + &
     &                (v_air(i_node) - vv2(i_node, sfc_lev))**2 )
#else /* SELFE */
     &          sqrt( (u_air(i_node) - uu2(sfc_lev,i_node))**2 + &
     &                (v_air(i_node) - vv2(sfc_lev,i_node))**2 )
#endif /* SELFE */
          if (speed_res .gt. 0.0) then
            tau = rho_air * u_star * u_star * speed_res / speed
            tau_xz(i_node) = - tau &
#ifndef SELFE
     &                     * (u_air(i_node) - uu2(i_node, sfc_lev)) &
#else /* SELFE */
     &                     * (u_air(i_node) - uu2(sfc_lev,i_node)) &
#endif /* SELFE */
     &                     / speed_res
            tau_yz(i_node) = - tau &
#ifndef SELFE
     &                     * (v_air(i_node) - vv2(i_node, sfc_lev)) &
#else /* SELFE */
     &                     * (v_air(i_node) - vv2(sfc_lev,i_node)) &
#endif /* SELFE */
     &                     / speed_res
          else
            tau_xz(i_node) = 0.0
            tau_yz(i_node) = 0.0
          endif

          if (mod(i_node-1,printit) .eq. 0) then
            write(38,*) 'sen_flux, lat_flux = ', &
     &                   sen_flux(i_node), lat_flux(i_node)
            write(38,*) 'tau_xz, tau_yz = ', &
     &                   tau_xz(i_node), tau_yz(i_node)
          endif
#ifndef SELFE
!         write(38,*) tnd(i_node, sfc_lev), sen_flux(i_node),
#else /* SELFE */
!         write(38,*) tnd(sfc_lev,i_node), sen_flux(i_node),
#endif /* SELFE */
!    +                lat_flux(i_node)

! end of wet/dry block
        endif

! end of loop over points
        enddo

        write(38,*) 'exit turb_fluxes'

      return
      end
!-----------------------------------------------------------------------
!
! Calculate saturation vapor pressure using the eighth order relative
! error norm method of Flatau et al., J Applied Meteo, v31, p 1507,
! Dec 1992.
!
      real*4 function esat_flat_r(t)
        implicit none
        real*4 t
        real*4 c0, c1, c2, c3, c4, c5, c6, c7, c8, t_eff
        parameter ( &
     &        c0= 6.11583699e+02,  c1= 0.444606896e+02, &
     &        c2= 0.143177157e+01, c3= 0.264224321e-01, &
     &        c4= 0.299291081e-03, c5= 0.203154182e-05, &
     &        c6= 0.702620698e-08, c7= 0.379534310e-11, &
     &        c8=-0.321582393e-13)

! t     : temperature in K
! t_eff : effective temperature in C

        t_eff = max(-85.,t-273.16)

        esat_flat_r = c0+t_eff*(c1+t_eff*(c2+t_eff*(c3+t_eff*(c4+t_eff* &
     &                         (c5+t_eff*(c6+t_eff*(c7+t_eff*c8)))))))

      return
      end
!-----------------------------------------------------------------------
      real*8 function psi_m(zeta)
        implicit none
        real*8 zeta, chi, half_pi

        half_pi = 2.0 * atan(1.0)
        chi = (1.0 - 16.0 * zeta)**0.25
        psi_m = 2.0 * log( 0.5 * (1.0 + chi) ) + &
     &          log( 0.5 * (1.0 + chi*chi) ) - &
     &          2.0 * atan(chi) + half_pi

      return
      end
!-----------------------------------------------------------------------
      real*8 function psi_h(zeta)
        implicit none
        real*8 zeta, chi

        chi = (1.0 - 16.0 * zeta)**0.25
        psi_h = 2.0 * log( 0.5 * (1.0 + chi*chi) )

      return
      end
!-----------------------------------------------------------------------
